package com.cnbm.processInspection.controller;

import com.cnbm.basic.dto.FactoryDTO;
import com.cnbm.basic.dto.UnitDTO;
import com.cnbm.basic.entity.ProductFeatures;
import com.cnbm.common.spc.math.StandardDiviation;
import com.cnbm.common.spc.util.DataUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.vo.R;
import com.cnbm.influx.config.InfluxClient;
import com.cnbm.influx.constant.Constant;
import com.cnbm.influx.param.PageInfo;
import com.cnbm.influx.param.QueryDataParam;
import com.cnbm.influx.param.Range;
import com.cnbm.influx.param.Tag;
import com.cnbm.influx.template.Event;
import com.cnbm.processInspection.dto.GraphArg;
import com.cnbm.processInspection.dto.InterpretationListArg;
import com.cnbm.processInspection.dto.XbarSGraphData;
import com.cnbm.processInspection.graphAnalyzed.MeanStandardDeviationGraph;
import com.cnbm.qualityPlanning.common.StatisticalControlledTest;
import com.cnbm.qualityPlanning.entity.ControlLimit;
import com.cnbm.qualityPlanning.entity.Point;
import com.cnbm.qualityPlanning.entity.SpecificationLimit;
import com.cnbm.qualityPlanning.entity.XbarSPoint;
import com.influxdb.client.domain.WritePrecision;
import com.influxdb.query.FluxRecord;
import com.influxdb.query.FluxTable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.time.Instant;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

@RestController
@RequestMapping("/processInspection")
public class ProcessInspectionController {
    private static final Logger logger = LoggerFactory.getLogger(ProcessInspectionController.class);



    @PostMapping("/XbarSGraphTest")
    public R<XbarSGraphData> xbarSGraphTest() throws Exception {
        ProductFeatures productFeatures = new ProductFeatures();
        productFeatures.setSl(new Float(5));
        productFeatures.setUsl(new Float(10));
        productFeatures.setLsl(new Float(1));

        productFeatures.setName("LTWeight");
        productFeatures.setSampleSize(BigDecimal.TEN);
        MeanStandardDeviationGraph meanStandardDeviationGraph = new MeanStandardDeviationGraph(productFeatures);

        QueryDataParam queryDataParam = new QueryDataParam();
        queryDataParam.setMeasurement("Weight");
        queryDataParam.setRange(new Range(DataUtils.getBeforeDate(10).toInstant(), Instant.now()));
        meanStandardDeviationGraph.initialDate(queryDataParam);

        XbarSGraphData xbarSGraphData = new XbarSGraphData(
                meanStandardDeviationGraph.getMsdGraphEntity(),
                meanStandardDeviationGraph.getXbarCL(),
                meanStandardDeviationGraph.getSCL(),
                meanStandardDeviationGraph.getSpecificationLimit(),
                meanStandardDeviationGraph.getProcessCapacity(),
                new StandardDiviation(meanStandardDeviationGraph.getXigma(),meanStandardDeviationGraph.getTotalXigma())
        );

        return R.ok("成功",xbarSGraphData);
    }


    //筛选条件顺序：
    //      ① 先工厂（产品表） => ② 检验类型（产品表）=> ③ 产品名（产品表）=> ④ 工序（产品表）=> ⑤ 特性（检验参数name）（ product_features 表） ；；
    //      ① 时间段（influxdb，必填）；② 样本大小（界面上可以设置，如果要自己设置，就要更新 ProductFeatures.name）
    //条件：1.检验时间段（influxdb里面） ;  7.样本大小（这个可选，也可以 用默认，产品那里配置。）
    // 2.工厂（先放一下） ;
    // 3.检验类型（产品表） ; 4.产品名 （产品表）; 5.工艺流程（就是工序 ，也是设备名，用到工序表）; 6.特性(product_features.name ,也是influxdb中的argName) ;
    @PostMapping("/XbarSGraph")
    public R<XbarSGraphData> xbarSGraph(@RequestBody GraphArg graphArg) throws Exception {

        ProductFeatures productFeatures = graphArg.getProductFeatures();

        MeanStandardDeviationGraph meanStandardDeviationGraph = new MeanStandardDeviationGraph(productFeatures);
        if(graphArg.getInterpretationScheme()!=null){
            meanStandardDeviationGraph.isNeedInterpretation(graphArg.getInterpretationScheme());
        }
        QueryDataParam queryDataParam = new QueryDataParam();
        queryDataParam.setMeasurement(Constant.measurement);
        queryDataParam.setRange(new Range(graphArg.getBegin().toInstant(), graphArg.getEnd().toInstant()));
        meanStandardDeviationGraph.initialDate(queryDataParam);

        XbarSGraphData xbarSGraphData = new XbarSGraphData(
                meanStandardDeviationGraph.getMsdGraphEntity(),
                meanStandardDeviationGraph.getXbarCL(),
                meanStandardDeviationGraph.getSCL(),
                meanStandardDeviationGraph.getSpecificationLimit(),
                meanStandardDeviationGraph.getProcessCapacity(),
                new StandardDiviation(meanStandardDeviationGraph.getXigma(),meanStandardDeviationGraph.getTotalXigma())
        );

        return R.ok("成功",xbarSGraphData);
    }

}
